%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MAIN_RFF.M estimates RFF using Auditory-SWIPE' and accounts for sample
% characteristics using the acoustic measure, pitch strength.
%
% Author: Jenny Vojtech (jmvo@bu.edu)
%         Stepp Lab, Boston University, MA
% Date:   04/18/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% DO NOT CHANGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all; clear; clc;

addpath('./dependencies/');
addpath('./dependencies/swipe');

global min_pitch_set;
global max_pitch_set;
global maxpkn;
global perform_fol;
global stimulicomb;

str_step1 = 'Finding F0 contour and pitch strength contour... ';
str_step2 = 'Finding fricative locations... ';
str_step3 = 'Finding cycles and calculating RFF... ';
waitbar_strings = {str_step1,str_step2,str_step3};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% SET THE FOLLOWING PARAMETERS: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
min_pitch_set = 50;    %default min pitch is 50 Hz
max_pitch_set = 400;   %default max pitch is 400 Hz
maxpkn        = 3;     %initially assume that there will be 3 VCV instances
stimulicomb   = 1;     %if RFF stimuli (e.g., /afa/, /ifi/, ufu/) are
%recorded separately, but you want the results
%combined into one file under the speaker ID, then
%set stimulicomb to 1; otherwise, set it to 0
folname       = 'C:\Users\jmvo\Desktop\aRFF\audio\';   %audio file location
perform_fol   = 'C:\Users\jmvo\Desktop\aRFF\perform\'; %performance folder
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% DO NOT CHANGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
save_progress = [perform_fol, 'progress.mat'];
allfiles      = dir([folname '*.wav']);
if exist(save_progress, 'file')
    load(save_progress);
else
    steps = zeros(3,1);
    RFFout_check = zeros(numel(allfiles),1);
end

for k = 1:size(steps,1)
    g = waitbar(0,waitbar_strings{k});
    if steps(k) < numel(allfiles)
        if steps(k) == 0
            start = 1;
        else
            start = steps(k)+1;
        end
        for filei = start:length(allfiles)
            waitbar(filei/length(allfiles),g,sprintf([waitbar_strings{k}...
                '%i/%i'],filei,length(allfiles)));
            filename = allfiles(filei).name;
            switch k
                case 1 %find the F0 & pitch strength contours and SNR
                    find_contours_aswipep(folname,filename);
                case 2 %find the location of fricatives
                    find_fric_aswipep(folname,filename);
                case 3 %find glottal cycles for RFF estimation
                    find_RFFcycles_aswipep(folname,filename,false);
            end
            RFFout_check(filei) = k;
            steps(k) = filei;
            save(save_progress,'steps','RFFout_check');
        end
    end
    close(g);
end
RFFoutput(allfiles(RFFout_check == size(steps,1)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% RFF REJECTION LEGEND %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%0 = not rejected
%1 = too few periodic cycles (pre-process rejection)
%2 = fails to reach steady state or unstable
%3 = glottalized
%4 = aperiodic or irregular
%5 = sharp transition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
